# Creates and populates two set[str]: all_files, all_dirs
import os
import micropython
@micropython.native
def __belay_hf(fn, buf):
    # inherently is inherently modulo 32-bit arithmetic
    @micropython.viper
    def xor_mm(data, state: uint, prime: uint) -> uint:
        for b in data:
            state = uint((state ^ uint(b)) * prime)
        return state

    h = 0x811c9dc5
    try:
        f = open(fn, "rb")
        while True:
            n = f.readinto(buf)
            if n == 0:
                break
            h = xor_mm(buf[:n], h, 0x01000193)
        f.close()
    except OSError:
        h = 0
    return h
def __belay_hfs(fns):
    buf = memoryview(bytearray(4096))
    print("_BELAYR" + repr([__belay_hf(fn, buf) for fn in fns]))
def __belay_mkdirs(fns):
    for fn in fns:
        try:
            os.mkdir(fn)
        except OSError:
            pass
all_files, all_dirs = set(), []
def __belay_fs(path="/", check=True):
    if not path:
        path = "/"
    elif not path.endswith("/"):
        path += "/"
    if check:
        try:
            os.stat(path)
        except OSError:
            return
    for elem in os.ilistdir(path):
        full_name = path + elem[0]
        if elem[1] & 0x4000:  # is_dir
            all_dirs.append(full_name)
            __belay_fs(full_name, check=False)
        else:
            all_files.add(full_name)



for file in all_files:
    os.remove(file)
for folder in reversed(all_dirs):
    try:
        os.rmdir(folder)
    except OSError:
        pass
del all_files, all_dirs, __belay_hf, __belay_hfs, __belay_mkdirs



import sys
def __belay(name):
    def inner(f):
        def func_wrapper(*args, **kwargs):
            res = f(*args, **kwargs)
            print("_BELAYR" + repr(res))
            return res
        def gen_wrapper(*args, **kwargs):
            for res in f(*args, **kwargs):
                print("_BELAYR" + repr(res))
                yield res
        globals()["_belay_" + name] = gen_wrapper if isinstance(f, type(lambda: (yield))) else func_wrapper
    return inner
def __belay_gen_next(x):
    try:
        next(x)
    except StopIteration:
        print("_BELAYS")



from functools import partial
from pathlib import Path
from time import sleep
from typing import List, Optional, Union

import typer
from rich.console import Console
from rich.progress import Progress
from typer import Argument, Option

from belay import Device

Arg = partial(Argument, ..., show_default=False)
Opt = partial(Option)

app = typer.Typer()
state = {}
console: Console

_help_port = (
    "Port (like /dev/ttyUSB0) or WebSocket (like ws://192.168.1.100) of device."
)
_help_password = (  # nosec
    "Password for communication methods (like WebREPL) that require authentication."
)


@app.callback()
def callback(silent: bool = False):
    """Tool to interact with MicroPython hardware."""
    global console, Progress
    state["silent"] = silent
    console_kwargs = {}
    if state["silent"]:
        console_kwargs["quiet"] = True
    console = Console(**console_kwargs)
    Progress = partial(Progress, console=console)


@app.command()
def sync(
    port: str = Arg(help=_help_port),
    folder: Path = Arg(help="Path of local file or folder to sync."),
    dst: str = Opt("/", help="Destination directory to unpack folder contents to."),
    password: str = Opt("", help=_help_password),
    keep: Optional[List[str]] = Opt(None, help="Files to keep."),
    ignore: Optional[List[str]] = Opt(None, help="Files to ignore."),
    mpy_cross_binary: Optional[Path] = Opt(
        None, help="Compile py files with this executable."
    ),
):
    """Synchronize a folder to device."""
    # Typer issues: https://github.com/tiangolo/typer/issues/410
    keep = keep if keep else None
    ignore = ignore if ignore else None

    with Progress() as progress:
        task_id = progress.add_task("")
        progress_update = partial(progress.update, task_id)
        progress_update(description=f"Connecting to {port}")
        device = Device(port, password=password)
        progress_update(description=f"Connected to {port}.")

        device.sync(
            folder,
            dst=dst,
            keep=keep,
            ignore=ignore,
            mpy_cross_binary=mpy_cross_binary,
            progress_update=progress_update,
        )

        progress_update(description="Sync complete.")


@app.command()
def run(
    port: str = Arg(help=_help_port),
    file: Path = Arg(help="File to run on-device."),
    password: str = Opt("", help=_help_password),
):
    """Run file on-device."""
    device = Device(port, password=password)
    content = file.read_text()
    device(content)


@app.command()
def exec(
    port: str = Arg(help=_help_port),
    statement: str = Arg(help="Statement to execute on-device."),
    password: str = Opt("", help=_help_password),
):
    """Execute python statement on-device."""
    device = Device(port, password=password)
    device(statement)


@app.command()
def info(
    port: str = Arg(help=_help_port),
    password: str = Opt("", help=_help_password),
):
    """Display device firmware information."""
    device = Device(port, password=password)
    version_str = "v" + ".".join(str(x) for x in device.implementation.version)
    print(
        f"{device.implementation.name} {version_str} - {device.implementation.platform}"
    )
    return device


@app.command()
def identify(
    port: str = Arg(help=_help_port),
    pin: int = Arg(help="GPIO pin to flash LED on."),
    password: str = Opt("", help=_help_password),
    neopixel: bool = Option(False, help="Indicator is a neopixel."),
):
    """Display device firmware information and blink an LED."""
    device = info(port=port, password=password)

    if device.implementation.name == "circuitpython":
        device(f"led = digitalio.DigitalInOut(board.GP{pin})")
        device("led.direction = digitalio.Direction.OUTPUT")

        if neopixel:
            device("from neopixel_write import neopixel_write")

            @device.task
            def set_led(state):
                val = (255, 255, 255) if state else (0, 0, 0)
                val = bytearray(val)
                neopixel_write(led, val)  # noqa: F821

        else:

            @device.task
            def set_led(state):
                led.value = state  # noqa: F821

    else:
        device(f"led = Pin({pin}, Pin.OUT)")

        if neopixel:
            device("import neopixel")
            device("pixel = neopixel.NeoPixel(led, 1)")

            @device.task
            def set_led(state):
                pixel[0] = (255, 255, 255) if state else (0, 0, 0)  # noqa: F821
                pixel.write()  # noqa: F821

        else:

            @device.task
            def set_led(state):
                led.value(state)  # noqa: F821

    while True:
        set_led(True)
        sleep(0.5)
        set_led(False)
        sleep(0.5)
